/**
 * name: @tresjs/post-processing
 * version: v0.5.0
 * (c) 2023
 * description: Post-processing library for TresJS
 * author: Alvaro Saburido <hola@alvarosaburido.dev> (https://github.com/alvarosabu/)
 */
import { inject as $, shallowRef as w, watchEffect as m, onUnmounted as T, watch as _, defineComponent as f, computed as k, provide as j, renderSlot as N } from "vue";
import { EffectPass as X, BloomEffect as x, GlitchEffect as b, GlitchMode as Y, OutlineEffect as y, PixelationEffect as P, DepthOfFieldEffect as F, EffectComposer as R, RenderPass as H, NormalPass as I, DepthDownsamplingPass as U } from "postprocessing";
import { useTresContext as S, normalizeColor as C, useRenderLoop as K } from "@tresjs/core";
import { HalfFloatType as O } from "three";
const A = Symbol(), v = (a) => {
  const t = $(A), e = w(null), n = w(null), { scene: o, camera: l } = S();
  m(() => {
    !l.value || !(n != null && n.value) || (n.value.mainCamera = l.value);
  });
  let s = () => {
  };
  return s = m(() => {
    !l.value || !(t != null && t.value) || !o.value || (s(), !n.value && (n.value = a(), e.value = new X(l.value, n.value), t.value.addPass(e.value)));
  }), T(() => {
    var r, i, d;
    e.value && ((r = t == null ? void 0 : t.value) == null || r.removePass(e.value)), (i = n.value) == null || i.dispose(), (d = e.value) == null || d.dispose();
  }), {
    pass: e,
    effect: n
  };
}, M = /([^[.\]])+/g, q = (a, t) => {
  if (!t)
    return;
  const e = Array.isArray(t) ? t : t.match(M);
  return e == null ? void 0 : e.reduce((n, o) => n && n[o], a);
}, z = (a, t, e) => {
  const n = Array.isArray(t) ? t : t.match(M);
  n && n.reduce((o, l, s) => (o[l] === void 0 && (o[l] = {}), s === n.length - 1 && (o[l] = e), o[l]), a);
}, J = (a, t) => {
  const e = { ...a };
  return t.forEach((n) => delete e[n]), e;
}, G = (a, t, e, n, o = {}) => _(
  a,
  (l) => {
    var s;
    if (t.value)
      if (l === void 0) {
        const r = n();
        z(t.value, e, q(r, e)), (s = r.dispose) == null || s.call(r);
      } else
        z(t.value, e, a());
  },
  o
), E = (a, t, e) => a.map(([n, o]) => G(
  n,
  t,
  o,
  e
)), L = (a, t, e) => Object.keys(a).map((n) => G(
  () => a[n],
  t,
  n,
  e
)), te = /* @__PURE__ */ f({
  __name: "Bloom",
  props: {
    blendFunction: {},
    intensity: {},
    kernelSize: {},
    luminanceThreshold: {},
    luminanceSmoothing: {},
    mipmapBlur: { type: Boolean, default: void 0 }
  },
  setup(a, { expose: t }) {
    const e = a, { pass: n, effect: o } = v(() => new x(e));
    return t({ pass: n, effect: o }), E(
      [
        // blendFunction is not updated, because it has no setter in BloomEffect
        [() => e.intensity, "intensity"],
        [() => e.kernelSize, "kernelSize"],
        [() => e.luminanceSmoothing, "luminanceMaterial.smoothing"],
        [() => e.luminanceThreshold, "luminanceMaterial.threshold"]
      ],
      o,
      () => new x()
    ), (l, s) => null;
  }
}), ne = /* @__PURE__ */ f({
  __name: "Glitch",
  props: {
    blendFunction: {},
    delay: {},
    duration: {},
    strength: {},
    mode: {},
    active: { type: Boolean },
    ratio: {},
    columns: {},
    chromaticAberrationOffset: {},
    perturbationMap: {},
    dtSize: {}
  },
  setup(a, { expose: t }) {
    const e = a, { pass: n, effect: o } = v(() => new b(e));
    return t({ pass: n, effect: o }), m(() => {
      const l = () => {
        if (e.mode !== void 0)
          return e.active === !1 ? Y.DISABLED : e.mode;
        const s = new b(), r = s.mode;
        return s.dispose(), r;
      };
      o.value && (o.value.mode = l());
    }), L(
      J(e, ["active", "mode", "blendFunction"]),
      o,
      () => new b()
    ), (l, s) => null;
  }
}), oe = /* @__PURE__ */ f({
  __name: "Outline",
  props: {
    outlinedObjects: {},
    blur: { type: Boolean, default: void 0 },
    xRay: { type: Boolean, default: void 0 },
    kernelSize: {},
    pulseSpeed: {},
    resolutionX: {},
    resolutionY: {},
    edgeStrength: {},
    patternScale: {},
    multisampling: {},
    blendFunction: {},
    patternTexture: {},
    resolutionScale: {},
    hiddenEdgeColor: {},
    visibleEdgeColor: {}
  },
  setup(a, { expose: t }) {
    const e = a, n = (c) => c !== void 0 ? C(c).getHex() : void 0, { camera: o, scene: l } = S(), s = {
      blur: e.blur,
      xRay: e.xRay,
      kernelSize: e.kernelSize,
      pulseSpeed: e.pulseSpeed,
      resolutionX: e.resolutionX,
      resolutionY: e.resolutionY,
      patternScale: e.patternScale,
      edgeStrength: e.edgeStrength,
      blendFunction: e.blendFunction,
      multisampling: e.multisampling,
      patternTexture: e.patternTexture,
      resolutionScale: e.resolutionScale,
      hiddenEdgeColor: n(e.hiddenEdgeColor),
      visibleEdgeColor: n(e.visibleEdgeColor)
    }, { pass: r, effect: i } = v(() => new y(l.value, o.value, s));
    t({ pass: r, effect: i }), _(
      [() => e.outlinedObjects, i],
      // watchEffect is intentionally not used here as it would result in an endless loop
      () => {
        var c;
        (c = i.value) == null || c.selection.set(e.outlinedObjects || []);
      },
      {
        immediate: !0
      }
    );
    const d = k(() => ({
      hiddenEdgeColor: e.hiddenEdgeColor ? C(e.hiddenEdgeColor) : void 0,
      visibleEdgeColor: e.visibleEdgeColor ? C(e.visibleEdgeColor) : void 0
    }));
    return E(
      [
        /* some properties are not updated because of different reasons:
            resolutionX - has no setter in OutlineEffect
            resolutionY - has no setter in OutlineEffect
            blendFunction - has no setter in OutlineEffect
            patternTexture - different type in constructor and in setter
            resolutionScale - has no setter in OutlineEffect
          */
        [() => e.blur, "blur"],
        [() => e.xRay, "xRay"],
        [() => e.pulseSpeed, "pulseSpeed"],
        [() => e.kernelSize, "kernelSize"],
        [() => e.edgeStrength, "edgeStrength"],
        [() => e.patternScale, "patternScale"],
        [() => e.multisampling, "multisampling"],
        [() => d.value.hiddenEdgeColor, "hiddenEdgeColor"],
        [() => d.value.visibleEdgeColor, "visibleEdgeColor"]
      ],
      i,
      () => new y()
    ), (c, B) => null;
  }
}), le = /* @__PURE__ */ f({
  __name: "Pixelation",
  props: {
    granularity: {}
  },
  setup(a, { expose: t }) {
    const e = a, { pass: n, effect: o } = v(() => new P(e.granularity));
    return t({ pass: n, effect: o }), L(
      e,
      o,
      () => new P()
    ), (l, s) => null;
  }
}), ae = /* @__PURE__ */ f({
  __name: "DepthOfField",
  props: {
    blendFunction: {},
    worldFocusDistance: {},
    worldFocusRange: {},
    focusDistance: {},
    focusRange: {},
    bokehScale: {},
    resolutionScale: {},
    resolutionX: {},
    resolutionY: {}
  },
  setup(a, { expose: t }) {
    const e = a, { camera: n } = S(), { pass: o, effect: l } = v(() => new F(n.value, e));
    return t({ pass: o, effect: l }), E(
      [
        // blendFunction is not updated, because it has no setter in BloomEffect
        [() => e.worldFocusDistance, "circleOfConfusionMaterial.worldFocusDistance"],
        [() => e.focusDistance, "circleOfConfusionMaterial.focusDistance"],
        [() => e.worldFocusRange, "circleOfConfusionMaterial.worldFocusRange"],
        [() => e.focusRange, "circleOfConfusionMaterial.focusRange"],
        [() => e.bokehScale, "bokehScale"],
        [() => e.resolutionScale, "blurPass.resolution.scale"],
        [() => e.resolutionX, "resolution.width"],
        [() => e.resolutionY, "resolution.height"]
      ],
      l,
      () => new F()
    ), (s, r) => null;
  }
});
let p;
function D() {
  var a;
  if (p !== void 0)
    return p;
  try {
    let t;
    const e = document.createElement("canvas");
    return p = !!(window.WebGL2RenderingContext && (t = e.getContext("webgl2"))), t && ((a = t.getExtension("WEBGL_lose_context")) == null || a.loseContext()), p;
  } catch {
    return p = !1;
  }
}
const se = /* @__PURE__ */ f({
  __name: "EffectComposer",
  props: {
    enabled: { type: Boolean, default: !0 },
    children: {},
    depthBuffer: { type: Boolean, default: void 0 },
    disableNormalPass: { type: Boolean, default: !1 },
    stencilBuffer: { type: Boolean, default: void 0 },
    resolutionScale: {},
    autoClear: { type: Boolean, default: !0 },
    multisampling: { default: 0 },
    frameBufferType: { default: O }
  },
  setup(a) {
    const t = a, { scene: e, camera: n, renderer: o, sizes: l } = S(), s = w(null);
    let r = null, i = null;
    j(A, s);
    const d = () => {
      s.value && (i = new I(e.value, n.value), i.enabled = !1, s.value.addPass(i), t.resolutionScale !== void 0 && D() && (r = new U({
        normalBuffer: i.texture,
        resolutionScale: t.resolutionScale
      }), r.enabled = !1, s.value.addPass(r)));
    };
    m(() => {
      s.value && l.width.value && l.height.value && s.value.setSize(l.width.value, l.height.value);
    });
    const c = k(() => {
      const u = new R(), g = {
        depthBuffer: t.depthBuffer !== void 0 ? t.depthBuffer : u.inputBuffer.depthBuffer,
        stencilBuffer: t.stencilBuffer !== void 0 ? t.stencilBuffer : u.inputBuffer.stencilBuffer,
        multisampling: D() ? 0 : t.multisampling !== void 0 ? t.multisampling : u.multisampling,
        frameBufferType: t.frameBufferType !== void 0 ? t.frameBufferType : O
      };
      return u.dispose(), g;
    }), B = () => {
      !o.value && !e.value && !n.value || (s.value = new R(o.value, c.value), s.value.addPass(new H(e.value, n.value)), t.disableNormalPass || d());
    };
    let h = () => {
    };
    h = _([l.height, l.width], () => {
      !l.height.value && !l.width.value || (m(B), h == null || h());
    }, {
      immediate: !0
    });
    const { onLoop: W } = K();
    return W(({ delta: u }) => {
      if (t.enabled && o.value && s.value && l.width.value && l.height.value) {
        const g = o.value.autoClear;
        o.value.autoClear = t.autoClear, t.stencilBuffer && !t.autoClear && o.value.clearStencil(), s.value.render(u), o.value.autoClear = g;
      }
    }), T(() => {
      var u;
      (u = s.value) == null || u.dispose();
    }), (u, g) => N(u.$slots, "default");
  }
});
export {
  te as Bloom,
  ae as DepthOfField,
  se as EffectComposer,
  ne as Glitch,
  oe as Outline,
  le as Pixelation
};
